/* Program to generate IP data file */
#define ProgName "ipcmp2rax"
#define ProgInfo "Generate R-AXIS Image File for Compton Scattering Correction"
/*                                                        11/13/2001  */
/*         for ANSI C                                                 */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v0.15_5/12/2005"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <fcntl.h>
#include <math.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"
#include "libxray.h"

#define MAXARG 14
#define MINARG 5
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define FLAT 0
#define CURVE_V 1
#define CURVE_H 2
#define BACK 3
#define W_FACTOR 0.02426

#ifdef _OLDHP
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
	double correctFlat();
	double correctCurveV();
	double correctCurveH();
	double correctBack();
	double (*correctf)();
#else
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1);
	double correctFlat(const double *z, const double *x, double *qval2x);
	double correctCurveV(const double *z, const double *x, double *qval2x);
	double correctCurveH(const double *z, const double *x, double *qval2x);
	double correctBack(const double *z, const double *x, double *qval2x);
	double (*correctf)(const double *z, const double *x, double *qval2x);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr1, *datPtr0;
	long int i, j, k, ipHeaderSize, ipHeaderSize2VER2,
	     maxInt, atomNum, recordLengthCreate, ipHeaderSizeMin,
	     flushWidth, flushWidth0;
	char file1[FNAMELEN], orgStr[256], cpuIntelFlag, drtxDoneFlag, drtzDoneFlag,
	     confirmBuffer[8], inputBuffer[20], argNum, arg0[4], *cPtr,
	     verboseFlag, shiftBit, xsizDoneFlag, zsizDoneFlag,
	     xpxlDoneFlag, zpxlDoneFlag, camrDoneFlag, lambdaDoneFlag,
	     magDoneFlag, cameraTypeDoneFlag, cameraTypeFlag,
	     wlngMode, file3[FNAMELEN], useRefFlag, ipHeaderSwapFlag;
	double rato, arg1, intensityDbl, mag, fact1x, fact1z, fact2, fact3,
	       zDist, xDist, I_inc, qval2, correct, lambda, res;
	struct ipHead ipHeader;
	time_t timeNow;
	FILE *fp1, *fp3;
	struct atomInc {
		char *atom;
		long int atomType;
		long int num;
	} *atomInc;
#define SIZE_OF_ATOM_INC sizeof(struct atomInc)
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	xsizDoneFlag = zsizDoneFlag = drtxDoneFlag = drtzDoneFlag
	 = xpxlDoneFlag = zpxlDoneFlag = camrDoneFlag = lambdaDoneFlag
	 = magDoneFlag = cameraTypeDoneFlag = cameraTypeFlag = wlngMode
	 = useRefFlag = 0;
	ipHeaderSwapFlag = 0;
	verboseFlag = 1;
	rato = 8.0;
	shiftBit = 3;
	maxInt = ((long)(0x7fff)) << shiftBit;
	ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
	initXrayAtom();
	qval2 = 0.0;
	mag = 1.0;
	flushWidth0 = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Argument */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strncpy(orgStr, argv[0], 254);
		strcat(orgStr, "/");
		strncpy(file1, argv[argc-1], FNAMELEN-1);
		for (i=1; i<argc-1; i++) {
			if (!strncmp(argv[i], "-r:", 3)) {
				if (useRefFlag) {
					fprintf(stderr, "Too much REFFILEs.\n");
					exit(EXIT_FAILURE);
				}
				if (strlen(argv[i]) < 4) {
					fprintf(stderr, "\"REFFILE\" must be provided.\n");
					exit(EXIT_FAILURE);
				}
				strncpy(file3, argv[i]+3, FNAMELEN-1);
				useRefFlag++;
			}
		}
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* Interactive ? */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Do you like to use Reference File to get parameters for New File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			useRefFlag = 1;
			fprintf(stdout, "Enter Reference Filename : ");
			if (fscanf(stdin, "%s", file3) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				exit(EXIT_FAILURE);
			}
		}
	}
	
/* Define ipHeader and Initialize */
	if (useRefFlag) {	/* Read REFFILE */
		if ((fp3 = fopen(file3, "r")) == NULL) {
			fprintf(stderr, " Reference File Open Error.\n");
			exit(EXIT_FAILURE);
		}
		fread(&ipHeader, ipHeaderSize, 1, fp3);
		if (ftell(fp3) != ipHeaderSize) {
			fprintf(stderr, " Reference File Read Error.\n");
			fclose(fp3);
			exit(EXIT_FAILURE);
		}
		fclose(fp3);
		/* File Format Check */
		ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
		/* ipHeader.rlng is assumed to be less than 65536;
		   if each data has 16-bits, width of IP should less than 32768 pixels. */
		if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
			if (verboseFlag) {
				fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
				do {
					fprintf(stdout, "  Is it true ? [Y/N] : ");
					if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						exit(EXIT_FAILURE);
					}
				} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
				if (toupper(confirmBuffer[0]) == 'N') {
					fprintf(stderr, " Unable to continue. Stop.\n");
					exit(EXIT_FAILURE);
				}
				fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
			}
		}
		/* Swap Header */
		if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
		/* Get ipHeader Information and Check */
		if (ipHeader.zpxl != ipHeader.rnum) {
			ipHeader.zpxl = ipHeader.rnum;
			if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height in Reference File has been modified.\n");
		}
		if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) {
			ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
			if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width in Reference File has been modified.\n");
		}
		if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH)
		 || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)
		 || (ipHeader.drtx < 0.0) || (ipHeader.drtx > ipHeader.xpxl)
		 || (ipHeader.drtz < 0.0) || (ipHeader.drtz > ipHeader.zpxl)
		 || (ipHeader.xsiz <= 0.0) || (ipHeader.zsiz <= 0.0)
		 || (ipHeader.camr <= 0.0) || (ipHeader.wlng <= 0.0)) {
			fprintf(stderr, " Reference File Format is Incorrect.\n");
			exit(EXIT_FAILURE);
		}
		/* Clean Header */
		cleanIpHeader(&ipHeader);
		ipHeader.rato = rato;
		timeNow = time(NULL);
		strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&timeNow));
		strcpy(ipHeader.memo, ProgName);
		strcat(ipHeader.memo, "/");
		strcat(ipHeader.memo, ipHeader.date);
	}
	else {	/* Do not use REFFILE */
		memset(&ipHeader, 0, ipHeaderSize);
		strcpy(ipHeader.name, "R-AXIS-DS"); /* Name of the Device */
		strcpy(ipHeader.vers, "1.0-2.0"); /* Version Number */
		strcpy(ipHeader.crsy, "cubic"); /* Crystal System */
		ipHeader.alng = 10.0; /* a-axis length in A */
		ipHeader.blng = 10.0; /* b-axis length in A */
		ipHeader.clng = 10.0; /* c-axis length in A */
		ipHeader.alfa = 90.0; /* angle alpha */
		ipHeader.beta = 90.0; /* angle beta */
		ipHeader.gamm = 90.0; /* angle gamma */
		strcpy(ipHeader.spgr, "P23"); /* Space Group */
		ipHeader.mosc = 0.3; /* Mozaic Spread */
		strcpy(ipHeader.opnm, "rigaku"); /* Operator Name */
		strcpy(ipHeader.trgt, "Cu"); /* X-ray Target */
		strcpy(ipHeader.mcro, "None"); /* Monochromator Material */
		ipHeader.m2ta = 0.0; /* Angle of the monochromator in 2-theta */
		strcpy(ipHeader.colm, "0.05-0.10"); /* Collimator */
		strcpy(ipHeader.filt, "Ni"); /* Kb filter */
		ipHeader.vltg = 40.0; /* X-ray tube voltage */
		ipHeader.crnt = 100.0; /* X-ray tube current */
		strcpy(ipHeader.focs, "0.3x0.3"); /* X-ray focus */
		strcpy(ipHeader.optc, "Ni-filter"); /* X-ray optics */
		ipHeader.cyld = 0; /* Camera type */
		ipHeader.weis = 0.0; /* weisenberg oscillation */
		strcpy(ipHeader.mnax, "-b*"); /* Mount Axis */
		strcpy(ipHeader.bmax, "-a"); /* Beam Axis */
		ipHeader.phi0 = 0.0; /* phi0 */
		ipHeader.phis = 0.0; /* phi start */
		ipHeader.phie = 0.0; /* phi end */
		ipHeader.oscn = 0; /* Frequency of Oscillation */
		ipHeader.omga = 0.0; /* omega in theta */
		ipHeader.fkai = 0.0; /* xai */
		ipHeader.thta = 0.000; /* 2-theta */
		ipHeader.mu = 0.000; /* mu */
		ipHeader.ipst = 0; /* Line of start reading */
		ipHeader.ipnm = 1; /* Number of IP */
		ipHeader.rato = rato; /* Ratio of IP in High/Low */
		ipHeader.ft_1 = 0.0; /* fading time 1 */
		ipHeader.ft_2 = 0.0; /* fading time 2 */
		strcpy(ipHeader.host, "IRIS"); /* Type of Host Computer */
		strcpy(ipHeader.ip, "HR-3"); /* Type of IP */
		ipHeader.dr_x = 0; /* Data direction in horizontal */
		ipHeader.dr_z = 0; /* Data direction in vertical */
		ipHeader.drxz = 0; /* Data direction face/back */
		timeNow = time(NULL);
		strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&timeNow));
		strcat(ipHeader.memo, ProgName);
		strcat(ipHeader.memo, "/");
		strcat(ipHeader.memo, ipHeader.date);
	}
	
/* Reading Organization Definition */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter the Organization of the scattering material\n");
		fprintf(stdout, " Format is \"numner-of-the-type-of-atom/type:num/type:num/...\"\n");
		fprintf(stdout, " Ex. for ethane : \"2/C:2/H:6\" : ");
		if (fscanf(stdin, "%s", orgStr) == EOF) {
			fprintf(stderr, "\nError in getting strings. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((cPtr = strtok(orgStr, "/")) == NULL || !isdigitEXT(cPtr)) {
		fprintf(stderr, "Information Format About the Organization is incorrect.\n");
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	atomNum = atol(cPtr);
	if ((atomInc = malloc(SIZE_OF_ATOM_INC*atomNum)) == NULL) {
		fprintf(stderr, " Momery Allocation Error. Exit.\n");
		exit(EXIT_FAILURE);
	}
	for (i=0; i<atomNum; i++) {
		if ((atomInc[i].atom = strtok(NULL, ":")) == NULL || (cPtr = strtok(NULL, "/")) == NULL
		  || !isdigitEXT(cPtr) || (atomInc[i].num = atol(cPtr)) < 0) {
			fprintf(stderr, "Information Format About the Organization is incorrect.\n");
			showUsage(stderr);
			free(atomInc);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		atomInc[i].atomType = checkXrayAtom(atomInc[i].atom);
		if (atomInc[i].atomType == -1) {
			fprintf(stderr, " Error. Irregal or Not Supported Atom Specification : %s\n", atomInc[i].atom);
			free(atomInc);
			exit(EXIT_FAILURE);
		}
	}
	
/* Conversion Parameter */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Camera is Flat[F] , Curved[C] or Flat(Back)[B] ? : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(atomInc);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'F') && (toupper(confirmBuffer[0]) != 'C') && (toupper(confirmBuffer[0]) != 'B'));
		if (toupper(confirmBuffer[0]) == 'F') {
			fprintf(stdout, "  Camera is Flat.\n");
			cameraTypeFlag = FLAT;
		}
		else if (toupper(confirmBuffer[0]) == 'B') {
			fprintf(stdout, "  Camera is Flat, Back Type.\n");
			cameraTypeFlag = BACK;
			ipHeader.cyld = 2;
		}
		else {
			fprintf(stdout, "  Camera is Curved. Is Cylinder Axis\n");
			ipHeader.cyld = 1;
			do {
				fprintf(stdout, "    Horizontal[H] or Vertical[V] on the Screen ? : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					free(atomInc);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'H') && (toupper(confirmBuffer[0]) != 'V'));
			if (toupper(confirmBuffer[0]) == 'H') {
				fprintf(stdout, "  Camera axis is in X-direction.\n");
				cameraTypeFlag = CURVE_H;
			}
			else {
				fprintf(stdout, "  Camera axis is in Z-direction.\n");
				cameraTypeFlag = CURVE_V;
			}
		}
		if (useRefFlag) {
			fprintf(stdout, "Reference File has following Values :\n");
			fprintf(stdout, " ImageSize is %ld [pixel] x %ld [pixel], WaveLength is %10.6f [A]\n", ipHeader.xpxl, ipHeader.zpxl, ipHeader.wlng);
			fprintf(stdout, " PixelSize is %8.5f [mm] x %8.5f [mm], Camera Length is %8.2f [mm]\n", ipHeader.xsiz, ipHeader.zsiz, ipHeader.camr);
			fprintf(stdout, " Centre is ( %7.1f x %7.1f )\n", ipHeader.drtx, ipHeader.drtz);
		}
		do {
			fprintf(stdout, "Enter Size of Image in X-direction [pixel]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(atomInc);
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				xpxlDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.xpxl = atol(inputBuffer);
			if (ipHeader.xpxl <= 0 || ipHeader.xpxl > MAXWIDTH) continue;
			xpxlDoneFlag = 1;
		} while (!xpxlDoneFlag);
		do {
			fprintf(stdout, "Enter Size of Image in Z-direction [pixel]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(atomInc);
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				zpxlDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.zpxl = atol(inputBuffer);
			if (ipHeader.zpxl <= 0 || ipHeader.zpxl > MAXHEIGHT) continue;
			zpxlDoneFlag = 1;
		} while (!zpxlDoneFlag);
		do {
			fprintf(stdout, "Enter PixelSize in X-direction [mm]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(atomInc);
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				xsizDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.xsiz = atof(inputBuffer);
			if (ipHeader.xsiz <= 0.0) continue;
			xsizDoneFlag = 1;
		} while (!xsizDoneFlag);
		do {
			fprintf(stdout, "Enter PixelSize in Z-direction [mm]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(atomInc);
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				zsizDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.zsiz = atof(inputBuffer);
			if (ipHeader.zsiz <= 0.0) continue;
			zsizDoneFlag = 1;
		} while (!zsizDoneFlag);
		do {
			fprintf(stdout, "Enter Centre of Image in X-direction [pixel]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(atomInc);
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				drtxDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.drtx = atof(inputBuffer);
			if (ipHeader.drtx < 0.0 || ipHeader.drtx > ipHeader.xpxl) continue;
			drtxDoneFlag = 1;
		} while (!drtxDoneFlag);
		do {
			fprintf(stdout, "Enter Centre of Image in Z-direction [pixel]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(atomInc);
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				drtzDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.drtz = atof(inputBuffer);
			if (ipHeader.drtz < 0.0 || ipHeader.drtz > ipHeader.zpxl) continue;
			drtzDoneFlag = 1;
		} while (!drtzDoneFlag);
		do {
			fprintf(stdout, "Enter Camera Length [mm]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(atomInc);
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				camrDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.camr = atof(inputBuffer);
			if (ipHeader.camr <= 0.0) continue;
			camrDoneFlag = 1;
		} while (!camrDoneFlag);
		do {
			fprintf(stdout, "Enter WaveLength [A]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(atomInc);
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				lambdaDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.wlng = atof(inputBuffer);
			if (ipHeader.wlng <= 0.0) continue;
			lambdaDoneFlag = 1;
		} while (!lambdaDoneFlag);
		do {
			fprintf(stdout, "Enter Magnification to Generate Image : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(atomInc);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			mag = atof(inputBuffer);
			if (mag <= 0.0) continue;
			magDoneFlag = 1;
		} while (!magDoneFlag);
		do {
			fprintf(stdout, "Do you use effect of wavelength change ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				free(atomInc);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') wlngMode = 1;
	}
	else {
		for (i=1; i<argc-1; i++) {
			if (!strncmp(argv[i], "-r:", 3)) continue;
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-f")) {
					cameraTypeFlag = FLAT;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-cv")) {
					ipHeader.cyld = 1;
					cameraTypeFlag = CURVE_V;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-ch")) {
					ipHeader.cyld = 1;
					cameraTypeFlag = CURVE_H;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-bf")) {
					ipHeader.cyld = 2;
					cameraTypeFlag = BACK;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-w")) wlngMode++;
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-dx")) {
					if (arg1 <= 0.0) break;
					ipHeader.xsiz = arg1;
					xsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-dz")) {
					if (arg1 <= 0.0) break;
					ipHeader.zsiz = arg1;
					zsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-cl")) {
					if (arg1 <= 0.0) break;
					ipHeader.camr = arg1;
					camrDoneFlag++;
				}
				else if (!strcmp(arg0, "-x")) {
					ipHeader.xpxl = dToLong(arg1);
					if (ipHeader.xpxl <= 0 || ipHeader.xpxl > MAXWIDTH) break;
					xpxlDoneFlag++;
				}
				else if (!strcmp(arg0, "-z")) {
					ipHeader.zpxl = dToLong(arg1);
					if (ipHeader.zpxl <= 0 || ipHeader.zpxl > MAXHEIGHT) break;
					zpxlDoneFlag++;
				}
				else if (!strcmp(arg0, "-xc")) {
					if (arg1 < 0.0) break;
					ipHeader.drtx = arg1;
					drtxDoneFlag++;
				}
				else if (!strcmp(arg0, "-zc")) {
					if (arg1 < 0.0) break;
					ipHeader.drtz = arg1;
					drtzDoneFlag++;
				}
				else if (!strcmp(arg0, "-lm")) {
					if (arg1 <= 0.0) break;
					ipHeader.wlng = arg1;
					lambdaDoneFlag++;
				}
				else if (!strcmp(arg0, "-m")) {
					if (arg1 <= 0.0) break;
					mag = arg1;
					magDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			free(atomInc);
			exit(EXIT_FAILURE);
		}
		if ((xpxlDoneFlag > 1) || (zpxlDoneFlag > 1) || (xsizDoneFlag > 1)
		 || (zsizDoneFlag > 1) || (drtxDoneFlag > 1) || (drtzDoneFlag > 1)
		 || (camrDoneFlag > 1) || (lambdaDoneFlag > 1) || (magDoneFlag > 1)
		 || (cameraTypeDoneFlag > 1) || (wlngMode > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			free(atomInc);
			exit(EXIT_FAILURE);
		}
		if (((!xpxlDoneFlag || !zpxlDoneFlag || !xsizDoneFlag
		   || !zsizDoneFlag || !drtxDoneFlag || !drtzDoneFlag
		   || !camrDoneFlag || !lambdaDoneFlag) && !useRefFlag)
		 || !magDoneFlag || !cameraTypeDoneFlag) {
			fprintf(stderr, " Lack of Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			free(atomInc);
			exit(EXIT_FAILURE);
		}
		if (ipHeader.drtx > ipHeader.xpxl || ipHeader.drtz > ipHeader.zpxl) {
			fprintf(stderr, " Arguments out of Range.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			free(atomInc);
			exit(EXIT_FAILURE);
		}
	}
	if (cameraTypeFlag == CURVE_V && ((ipHeader.drtx - ipHeader.camr*M_PI/ipHeader.xsiz >= 0.0) || (ipHeader.drtx + ipHeader.camr*M_PI/ipHeader.xsiz <= ipHeader.xpxl))) {
		fprintf(stderr, " Curved Camera \"Width\" is longer than expected.\n");
		fprintf(stderr, "  Incident beam direction must be excluded.\n");
		exit(EXIT_FAILURE);
	}
	if (cameraTypeFlag == CURVE_H && ((ipHeader.drtz - ipHeader.camr*M_PI/ipHeader.zsiz >= 0.0) || (ipHeader.drtz + ipHeader.camr*M_PI/ipHeader.zsiz <= ipHeader.zpxl))) {
		fprintf(stderr, " Curved Camera \"Height\" is longer than expected.\n");
		fprintf(stderr, "  Incident beam direction must be excluded.\n");
		exit(EXIT_FAILURE);
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " Incoherent Scattering for ");
		for (i=0; i<atomNum; i++) fprintf(stdout, "%s%d", atomInc[i].atom, (int)(atomInc[i].num));
		fprintf(stdout, "\n");
		fprintf(stdout, " ImageSize is %ld [pixel] x %ld [pixel], WaveLength is %10.6f [A]\n", ipHeader.xpxl, ipHeader.zpxl, ipHeader.wlng);
		fprintf(stdout, " PixelSize is %8.5f [mm] x %8.5f [mm], Camera Length is %8.2f [mm]\n", ipHeader.xsiz, ipHeader.zsiz, ipHeader.camr);
		fprintf(stdout, " Centre is ( %7.1f x %7.1f )\n", ipHeader.drtx, ipHeader.drtz);
		fprintf(stdout, " Output Intensity is magnified by %10.2f times.\n", mag);
		if (cameraTypeFlag == FLAT) fprintf(stdout, " Camera is Flat.\n");
		else if (cameraTypeFlag == CURVE_V) fprintf(stdout, " Camera is Cylindrical along Z-direction.\n");
		else if (cameraTypeFlag == CURVE_H) fprintf(stdout, " Camera is Cylindrical along X-direction.\n");
		else if (cameraTypeFlag == BACK) fprintf(stdout, " Camera is Flat, with BackLaue mode.\n");
	}
	
/* Set Parameters to Create */
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(flushWidth, ipHeaderSizeMin);
	ipHeader.rlng = recordLengthCreate;
	ipHeader.rnum = ipHeader.zpxl;
	flushWidth0 = recordLengthCreate - flushWidth;
	fact1x = ipHeader.xsiz / ipHeader.camr;
	fact1z = ipHeader.zsiz / ipHeader.camr;
	fact2 = 2.0 / ipHeader.wlng / ipHeader.wlng;
	fact3 = ipHeader.xsiz * ipHeader.zsiz / ipHeader.camr / ipHeader.camr * mag;
	switch (cameraTypeFlag) {
		case FLAT:
			correctf = correctFlat;
			break;
		case BACK:
			correctf = correctFlat;
			break;
		case CURVE_V:
			correctf = correctCurveV;
			break;
		case CURVE_H:
			correctf = correctCurveH;
			break;
		default:
			break;
			/* NOT REACHED */
	}
	j = 0;
	for (i=0; i<atomNum; i++) {
		j += sprintf(inputBuffer, "%s%d", atomInc[i].atom, (int)(atomInc[i].num));
		if (j >= 20) break;
		strcat(ipHeader.crnm, inputBuffer);
	}; /* Crystal Name */
	ipHeader.fext = mag; /* Exposure Time in min. */

/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter  Output  Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			free(atomInc);
			exit(EXIT_FAILURE);
		}
	}
	while (createFileEXT(file1, &fp1)) {
		if (!verboseFlag) {
			fprintf(stderr, " Output File Creation Failure. Exit.\n");
			free(atomInc);
			exit(EXIT_FAILURE);
		}
		fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			free(atomInc);
			exit(EXIT_FAILURE);
		}
	}
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		remove(file1);
		free(atomInc);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Swap Header */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp1);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSize, 1, fp1);
	if (flushWidth0 != 0) free(datPtr0);
	
/* Swap Header again */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if (wlngMode) {
		for (i=0; i<ipHeader.rnum; i++) {
			if ((datPtr1 = malloc(ipHeader.rlng)) == NULL) {
				free(atomInc);
				if (flushWidth0 > 0) free(datPtr0);
				fprintf(stderr, " Failed to allocate memory.\n");
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					fseek(fp1, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp1);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp1));
					}
					fclose(fp1);
				}
				else {
					fclose(fp1);
					remove(file1);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			zDist = (i+0.5-ipHeader.drtz) * fact1z;
			for (j=0; j<ipHeader.xpxl; j++) {
				I_inc = 0.0;
				xDist = (j+0.5-ipHeader.drtx) * fact1x;
				correct = (*correctf)(&xDist, &zDist, &qval2);
				lambda = ipHeader.wlng + W_FACTOR * qval2;
				qval2 *= 2.0 / lambda / lambda;
				for (k=0; k<atomNum; k++) {
					if (calcXrayCompFactor(&qval2, &(atomInc[k].atomType), &res)) {
						fprintf(stderr, "Error in Calculating Compton Scattering Factor.\n");
						free(atomInc);
						if (flushWidth0 > 0) free(datPtr0);
						if (i) {
							ipHeader.zpxl = ipHeader.rnum = i;
							if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
							if (cpuIntelFlag) swapIpHeader(&ipHeader);
							fseek(fp1, 0, SEEK_SET);
							fwrite(&ipHeader, ipHeaderSize, 1, fp1);
							fprintf(stdout, " Output File is saved using available data.\n");
							if (cpuIntelFlag) swapIpHeader(&ipHeader);
							if (verboseFlag) {
								fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
								fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp1));
							}
							fclose(fp1);
						}
						else {
							fclose(fp1);
							remove(file1);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					I_inc += res * atomInc[k].num;
				}
				intensityDbl = I_inc * correct * fact3;
				datPtr1[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
			}
			fwrite(datPtr1, flushWidth, 1, fp1);
			if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp1);
			free(datPtr1);
		}
	}
	else {
		for (i=0; i<ipHeader.rnum; i++) {
			if ((datPtr1 = malloc(ipHeader.rlng)) == NULL) {
				free(atomInc);
				if (flushWidth0 > 0) free(datPtr0);
				fprintf(stderr, " Failed to allocate memory.\n");
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					fseek(fp1, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp1);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp1));
					}
					fclose(fp1);
				}
				else {
					fclose(fp1);
					remove(file1);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			zDist = (i+0.5-ipHeader.drtz) * fact1z;
			for (j=0; j<ipHeader.xpxl; j++) {
				I_inc = 0.0;
				xDist = (j+0.5-ipHeader.drtx) * fact1x;
				correct = (*correctf)(&xDist, &zDist, &qval2);
				qval2 *= fact2;
				for (k=0; k<atomNum; k++) {
					if (calcXrayCompFactor(&qval2, &(atomInc[k].atomType), &res)) {
						fprintf(stderr, "Error in Calculating Compton Scattering Factor.\n");
						free(atomInc);
						if (flushWidth0 > 0) free(datPtr0);
						if (i) {
							ipHeader.zpxl = ipHeader.rnum = i;
							if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
							if (cpuIntelFlag) swapIpHeader(&ipHeader);
							fseek(fp1, 0, SEEK_SET);
							fwrite(&ipHeader, ipHeaderSize, 1, fp1);
							fprintf(stdout, " Output File is saved using available data.\n");
							if (cpuIntelFlag) swapIpHeader(&ipHeader);
							if (verboseFlag) {
								fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
								fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp1));
							}
							fclose(fp1);
						}
						else {
							fclose(fp1);
							remove(file1);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					I_inc += res * atomInc[k].num;
				}
				intensityDbl = I_inc * correct * fact3;
				datPtr1[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
			}
			fwrite(datPtr1, flushWidth, 1, fp1);
			if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp1);
			free(datPtr1);
		}
	}
	
/* End */
	free(atomInc);
	if (flushWidth0 > 0) free(datPtr0);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp1));
	}
	fclose(fp1);
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
	fprintf(fp0, "        %s [-q|-v] orgStr {-f|-cv|-ch|-bf} -x:VALUE -z:VALUE\n", ProgName);
	fprintf(fp0, "          -xc:VALUE -zc:VALUE -dx:VALUE -dz:VALUE -cl:VALUE -lm:VALUE\n");
	fprintf(fp0, "          -m:VALUE [-w] file\n");
	fprintf(fp0, "        %s [-q|-v] orgStr -r:REFFILE {-f|-cv|-ch|-bf}\n", ProgName);
	fprintf(fp0, "          -m:VALUE [-w] file\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Generate R-AXIS Image File emulating\n");
	fprintf(fp0, "  Incoherent Scattering Intensities.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   orgStr ...... Organization of scattering material.\n");
	fprintf(fp0, "                 Format is \"numner-of-the-type-of-atom/type:num/type:num/...\"\n");
	fprintf(fp0, "                 Ex. for ethane : \"2/C:2/H:6\"\n");
	fprintf(fp0, "   -f .......... Assume Camera is Flat.\n");
	fprintf(fp0, "   -cv ......... Assume Camera is Cylindrical, whose axis vertical to screen.\n");
	fprintf(fp0, "   -ch ......... Assume Camera is Cylindrical, whose axis horizontal to screen.\n");
	fprintf(fp0, "   -bf ......... Assume Camera is Flat, with \"Back Laue\" Type.\n");
	fprintf(fp0, "                 \"Centre\" means incident.\n");
	fprintf(fp0, "   -x:VALUE .... Set Number of Pixel in X to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -z:VALUE .... Set Number of Pixel in Z to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -xc:VALUE ... Set Centre Position X to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -zc:VALUE ... Set Centre Position Z to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -dx:VALUE ... Set PixelSize in X-direction to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -dz:VALUE ... Set PixelSize in Z-direction to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -cl:VALUE ... Set Camera Langth to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -lm:VALUE ... Set wavelength to \"VALUE\" [A].\n");
	fprintf(fp0, "   -r:REFFILE\n");
	fprintf(fp0, "             ... Instead of setting file parameters in above options,\n");
	fprintf(fp0, "                 use parameters just same as \"REFFILE\".\n");
	fprintf(fp0, "                 \"REFFILE\" must be valid R-AXIS Image File.\n");
	fprintf(fp0, "   -m:VALUE .... Set magnification to the calculated intensity to \"VALUE\".\n");
	fprintf(fp0, "   -w .......... Use effect of wavelength change.\n");
	fprintf(fp0, "   file ........ Output R-AXIS Image Filename. This must be the last argument.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	double *arg1;
#else
char splitArg(const char *argV, char *arg0, double *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	return (2);
}

#ifdef _OLDHP
double correctFlat(x, z, qval2x)
	double *x, *z, *qval2x;
#else
double correctFlat(const double *x, const double *z, double *qval2x)
#endif
{
	/* z = z/camr , x = x/camr */
	double cosTheta;
	
	cosTheta = 1.0 / sqrt(1.0 + (*x)*(*x) + (*z)*(*z));
	if (((*qval2x) = 1.0 - cosTheta) == 0.0) return (0);
	return (cosTheta*cosTheta*cosTheta);
}

#ifdef _OLDHP
double correctCurveV(x, z, qval2x)
	double *x, *z, *qval2x;
#else
double correctCurveV(const double *x, const double *z, double *qval2x)
#endif
{
	double cosTheta, cosPhi;
	
	cosTheta = cos(*x) / sqrt(1.0 + (*z)*(*z));
	if (((*qval2x) = 1.0 - cosTheta) == 0.0) return (0);
	cosPhi = 1.0 / sqrt(1.0 + (*z)*(*z));
	return (cosPhi*cosPhi*cosPhi);
}

#ifdef _OLDHP
double correctCurveH(z, x, qval2x)
	double *x, *z, *qval2x;
#else
double correctCurveH(const double *z, const double *x, double *qval2x)
#endif
{
	double cosTheta, cosPhi;
	
	cosTheta = cos(*x) / sqrt(1.0 + (*z)*(*z));
	if (((*qval2x) = 1.0 - cosTheta) == 0.0) return (0);
	cosPhi = 1.0 / sqrt(1.0 + (*z)*(*z));
	return (cosPhi*cosPhi*cosPhi);
}

#ifdef _OLDHP
double correctBack(x, z, qval2x)
	double *x, *z, *qval2x;
#else
double correctBack(const double *x, const double *z, double *qval2x)
#endif
{
	double cosTheta;
	
	cosTheta = -1.0 / sqrt(1.0 + (*x)*(*x) + (*z)*(*z));
	if (((*qval2x) = 1.0 - cosTheta) == 0.0) return (0);
	return (cosTheta*cosTheta*cosTheta);
}
